@@ -114,7 +114,19 @@ except:
 import nrn
 import _neuron_section
 h  = hoc.HocObject()
+version = h.nrnversion(5)
+__version__ = version
 
+if not hasattr(hoc, "__file__"):
+  import platform
+  p = h.nrnversion(6)
+  if "--prefix=" in p:
+    p = p[p.find('--prefix=') + 9:]
+    p = p[:p.find("'")]
+  else:
+    p = "/usr/local/nrn"
+  p = p + "/%s/lib/libnrnpython.so"%platform.machine()
+  setattr(hoc, "__file__", p)
 
 # As a workaround to importing doc at neuron import time
 # (which leads to chicken and egg issues on some platforms)
@@ -143,6 +155,14 @@ def test():
     runner = unittest.TextTestRunner(verbosity=2)
     runner.run(neuron.tests.suite())
 
+def test_rxd():
+    """ Runs a tests on the rxd and crxd modules."""
+    import neuron.tests
+    import unittest
+
+    runner = unittest.TextTestRunner(verbosity=2)
+    runner.run(neuron.tests.test_rxd.suite())
+
 
 # ------------------------------------------------------------------------------
 # class factory for subclassing h.anyclass
@@ -166,7 +186,7 @@ def load_mechanisms(path):
 
     This function will not load a mechanism path twice.
 
-    The path should specify the directory in which nrnmodliv was run,
+    The path should specify the directory in which nrnivmodl or mknrndll was run,
     and in which the directory 'i686' (or 'x86_64' or 'powerpc' depending on your platform)
     was created"""
 
@@ -175,21 +195,29 @@ def load_mechanisms(path):
     global nrn_dll_loaded
     if path in nrn_dll_loaded:
         print("Mechanisms already loaded from path: %s.  Aborting." % path)
-        return
+        return True
     
     # in case NEURON is assuming a different architecture to Python,
     # we try multiple possibilities
 
+    libname = 'libnrnmech.so'
+    libsubdir = '.libs'
     arch_list = [platform.machine(), 'i686', 'x86_64', 'powerpc', 'umac']
 
+    # windows loads nrnmech.dll
+    if h.unix_mac_pc() == 3:
+        libname = 'nrnmech.dll'
+        libsubdir = ''
+        arch_list = ['']
+
     for arch in arch_list:
-        lib_path = os.path.join(path, arch, '.libs', 'libnrnmech.so')
+        lib_path = os.path.join(path, arch, libsubdir, libname)
         if os.path.exists(lib_path):
             h.nrn_load_dll(lib_path)
             nrn_dll_loaded.append(path)
-            return
+            return True
     print("NEURON mechanisms not found in %s." % path)
-
+    return False
 
 import os,sys
 if 'NRN_NMODL_PATH' in os.environ:
@@ -289,15 +317,16 @@ def psection(section):
 
     Print info about section in a hoc format which is executable.
     (length, parent, diameter, membrane information)
+    
+    Use section.psection() instead to get a data structure that
+    contains the same information and more.
 
     See:
 
-    http://neuron.yale.edu/neuron/docs/help/neuron/neuron/nrnoc.html#psection
+    https://www.neuron.yale.edu/neuron/static/py_doc/modelspec/programmatic/topology.html?#psection
 
     """
-    section.push()
-    h.psection()
-    h.pop_section()
+    h.psection(sec=section)
 
 def init():
     """
@@ -305,9 +334,10 @@ def init():
 
     Initialize the simulation kernel.  This should be called before a run(tstop) call.
 
-    Equivalent to hoc finitialize():
+    Use h.finitialize() instead, which allows you to specify the membrane potential
+    to initialize to; via e.g. h.finitialize(-65)
 
-    http://neuron.yale.edu/neuron/docs/help/neuron/neuron/nrnoc.html#finitialize
+    https://www.neuron.yale.edu/neuron/static/py_doc/simctrl/programmatic.html?#finitialize
     
     """
     h.finitialize()
@@ -427,15 +457,30 @@ def nrn_dll(printpath=False):
         the_dll = ctypes.cdll[hoc.__file__]
         return the_dll        
     except:
-      pass
+        pass
 
-    neuron_home = os.path.split(os.path.split(h.neuronhome())[0])[0]
 
     success = False
     if sys.platform == 'msys' or sys.platform == 'win32':
       p = 'hoc%d%d' % (sys.version_info[0], sys.version_info[1])
     else:
       p = 'hoc'
+
+    try:
+        # maybe hoc.so in this neuron module
+        base_path = os.path.join(os.path.split(__file__)[0], p)
+        dlls = glob.glob(base_path + '*.*')
+        for dll in dlls:
+            try:
+                the_dll = ctypes.cdll[dll]
+                if printpath : print(dll)
+                return the_dll
+            except:
+                pass
+    except:
+        pass
+    # maybe old default module location
+    neuron_home = os.path.split(os.path.split(h.neuronhome())[0])[0]
     base_path = os.path.join(neuron_home, 'lib' , 'python', 'neuron', p)
     for extension in ['', '.dll', '.so', '.dylib']:
         dlls = glob.glob(base_path + '*' + extension)
@@ -550,21 +595,42 @@ def _pkl(arg):
   #print 'neuron._pkl arg is ', arg
   return h.Vector(0)
 
-def nrnpy_pr(s):
-  sys.stdout.write(s.decode())
+def nrnpy_pass():
+  return 1
+
+def nrnpy_pr(stdoe, s):
+  if stdoe == 1:
+    sys.stdout.write(s.decode())
+  else:
+    sys.stderr.write(s.decode())
   return 0
 
+#flag to prevent both rxd and crxd being used
+_has_rxd = {'rxd':False,'crxd':False}
+
 if not embedded:
   try:
-    # callback in place of hoc printf
+    # nrnpy_pr callback in place of hoc printf
     # ensures consistent with python stdout even with jupyter notebook.
-    nrnpy_pr_proto = ctypes.CFUNCTYPE(ctypes.c_int, ctypes.c_char_p)
-    nrnpy_set_pr = nrn_dll_sym('nrnpy_set_pr')
-    nrnpy_set_pr.argtypes = [nrnpy_pr_proto]
+    # nrnpy_pass callback used by h.doNotify() in MINGW when not called from
+    # gui thread in order to allow the gui thread to run.
+
+    nrnpy_set_pr_etal = nrn_dll_sym('nrnpy_set_pr_etal')
+
+    nrnpy_pr_proto = ctypes.CFUNCTYPE(ctypes.c_int, ctypes.c_int, ctypes.c_char_p)
+    nrnpy_pass_proto = ctypes.CFUNCTYPE(ctypes.c_int)
+    nrnpy_set_pr_etal.argtypes = [nrnpy_pr_proto, nrnpy_pass_proto]
 
     nrnpy_pr_callback = nrnpy_pr_proto(nrnpy_pr)
-    nrnpy_set_pr(nrnpy_pr_callback)
+    nrnpy_pass_callback = nrnpy_pass_proto(nrnpy_pass)
+    nrnpy_set_pr_etal(nrnpy_pr_callback, nrnpy_pass_callback)
   except:
     print("Failed to setup nrnpy_pr")
     pass
 
+try:
+  from neuron.psection import psection
+  nrn.set_psection(psection)
+except:
+  print("Failed to setup nrn.Section.psection")
+pass
