@@ -3,7 +3,7 @@
 
 echo -e "\033]0;idraw\007"
 
-IDRAW="${NRNHOME}/../iv/${CPU}/bin/idraw"
+IDRAW="${NRNHOME}/../iv/x86_64/bin/idraw"
 
 cd
 "${IDRAW}" "$@"
