@@ -10,6 +10,8 @@ void cvode_finitialize();
 extern void (*nrn_multisplit_setup_)();
 }
 
+extern "C" int hoc_return_type_code;
+
 #include <math.h>
 #include <stdlib.h>
 #include "classreg.h"
@@ -119,6 +121,7 @@ static double spikestat(void* v) {
 	return 0;
 }
 static double queue_mode(void* v) {
+	hoc_return_type_code = 1; // integer
 #if BBTQ == 3 || BBTQ == 4
 	if (ifarg(1)) {
 		nrn_use_fifo_queue_ = chkarg(1, 0, 1) ? true : false;
@@ -211,6 +214,7 @@ static double active(void* v) {
 			d->re_init(nt_t);
 		}
 	}
+	hoc_return_type_code = 2; // boolean
 	return cvode_active_;
 }
 static double stiff(void* v) {
@@ -218,6 +222,7 @@ static double stiff(void* v) {
 	if (ifarg(1)) {
 		d->stiff((int)chkarg(1, 0, 2));
 	}
+	hoc_return_type_code = 1; // integer
 	return double(d->stiff());
 }
 static double maxorder(void* v) {
@@ -225,11 +230,13 @@ static double maxorder(void* v) {
 	if (ifarg(1)) {
 		d->maxorder((int)chkarg(1, 0, 5));
 	}
+	hoc_return_type_code = 1; // integer
 	return d->maxorder();
 }
 static double order(void* v) {
 	NetCvode* d = (NetCvode*)v;
 	int i = 0;
+	hoc_return_type_code = 1; // integer
 	if (ifarg(1)) {
 		// only thread 0
 		i = int(chkarg(1, 0, d->p->nlcv_ - 1));
@@ -303,6 +310,7 @@ static double statename(void* v) {
 
 static double use_local_dt(void* v) {
 	NetCvode* d = (NetCvode*)v;
+	hoc_return_type_code = 2; // boolean
 	if (ifarg(1)) {
 		int i = (int)chkarg(1, 0, 1);
 		d->localstep(i);
@@ -312,6 +320,7 @@ static double use_local_dt(void* v) {
 
 static double use_daspk(void* v) {
 	NetCvode* d = (NetCvode*)v;
+	hoc_return_type_code = 2; // boolean
 	if (ifarg(1)) {
 		int i = (int)chkarg(1, 0, 1);
 		if ((i != 0) != d->use_daspk()) {
@@ -333,6 +342,7 @@ static double dae_init_dteps(void* v) {
 
 static double use_mxb(void* v) {
 	NetCvode* d = (NetCvode*)v;
+	hoc_return_type_code = 2; // boolean
 	if (ifarg(1)) {
 		int i = (int)chkarg(1,0,1);
 		if (use_sparse13 != i) {
@@ -354,6 +364,7 @@ static double cache_efficient(void* v) {
 
 static double use_long_double(void* v) {
 	NetCvode* d = (NetCvode*)v;
+	hoc_return_type_code = 2; // boolean
 	if (ifarg(1)) {
 		int i = (int)chkarg(1,0,1);
 		d->use_long_double_ = i;
@@ -440,6 +451,7 @@ static double tstop_event(void* v) {
 
 static double current_method(void* v) {
 	NetCvode* d = (NetCvode*)v;
+	hoc_return_type_code = 1; // integer
 	int modeltype = nrn_modeltype();
 	int methodtype = secondorder; // 0, 1, or 2
 	int localtype = 0;
@@ -560,6 +572,7 @@ static double extra_scatter_gather_remove(void* v) {
 
 static double use_fast_imem(void* v) {
 	int i = nrn_use_fast_imem;
+	hoc_return_type_code = 2; // boolean
 	if (ifarg(1)) {
 		nrn_use_fast_imem = int(chkarg(1, 0., 1.));
 		nrn_fast_imem_alloc();
