@@ -15,7 +15,7 @@
 #include "classreg.h"
 
 typedef void (*ReceiveFunc)(Point_process*, double*, double);
-
+extern "C" int hoc_return_type_code;
 extern "C" {
 // from nrnoc
 #include "membfunc.h"
@@ -593,6 +593,7 @@ static double ms_get(void* v) {
 	return ((MechanismStandard*)v)->get(gargstr(1), i);
 }
 static double ms_count(void* v) {
+	hoc_return_type_code = 1;
 	return ((MechanismStandard*)v)->count();
 }
 static double ms_name(void* v) {
@@ -605,6 +606,7 @@ static double ms_name(void* v) {
    	n = ms->name();
    }
    hoc_assign_str(hoc_pgargstr(1), n);
+   hoc_return_type_code = 1;
    return double(rval);
 }
 
@@ -954,6 +956,7 @@ static double mt_selected(void* v) {
 	if (ifarg(1)) {
 		hoc_assign_str(hoc_pgargstr(1), mt->selected());
 	}
+	hoc_return_type_code = 1;
 	return double(i);
 }
 static double mt_internal_type(void* v) {
@@ -976,6 +979,7 @@ static double mt_remove(void* v) {
 }
 static double mt_count(void* v) {
 	MechanismType* mt = (MechanismType*)v;
+	hoc_return_type_code = 1;
 	return double(mt->count());
 }
 static double mt_menu(void* v) {
@@ -998,14 +1002,17 @@ static double mt_action(void* v) {
 }
 static double mt_is_target(void* v) {
 	MechanismType* mt = (MechanismType*)v;
+	hoc_return_type_code = 2;
 	return double(mt->is_netcon_target(int(chkarg(1, 0, mt->count()))));
 }
 static double mt_has_net_event(void* v) {
 	MechanismType* mt = (MechanismType*)v;
+	hoc_return_type_code = 2;
 	return double(mt->has_net_event(int(chkarg(1, 0, mt->count()))));
 }
 static double mt_is_artificial(void* v) {
 	MechanismType* mt = (MechanismType*)v;
+	hoc_return_type_code = 2;
 	return double(mt->is_artificial(int(chkarg(1, 0, mt->count()))));
 }
 static Object** mt_pp_begin(void* v) {
