@@ -5,10 +5,10 @@ extern char* nrn_version();
 
 /* change this to correspond to the ../nmodl/nocpout nmodl_version_ string*/
 static char nmodl_version_[] =
-"6.2.0";
+"7.5.0";
 
 static char banner[] =
-"Duke, Yale, and the BlueBrain Project -- Copyright 1984-2016\n\
+"Duke, Yale, and the BlueBrain Project -- Copyright 1984-2018\n\
 See http://neuron.yale.edu/neuron/credits\n";
 
 # include	<stdio.h>
@@ -374,6 +374,9 @@ if (strcmp(nrn_mech_dll, "nrnmech.dll") == 0) {
 #endif /* WIN32 || NRNMECH_DLL_STYLE */
 	s = hoc_lookup("section_owner");
 	s->type = OBJECTFUNC;
+
+	/* verify that all ions have a defined CHARGE */
+	nrn_verify_ion_charge_defined();
 }
 
 void initnrn(void) {
@@ -482,7 +485,7 @@ void nrn_register_mech_common(
 	
 /*printf("%s %s\n", m[0], m[1]);*/
 	if (strcmp(m[0], "0") == 0) { /* valid by nature */
-	}else if (m[0][0] != '6') { /* must be 5.1 or before */
+	}else if (m[0][0] > '9') { /* must be 5.1 or before */
 fprintf(stderr, "Mechanism %s needs to be re-translated.\n\
 It's pre version 6.0 \"c\" code is incompatible with this neuron version.\n", m[0]);
 		nrn_exit(1);
