@@ -70,6 +70,9 @@ extern int nrn_matrix_dim(void*, int);
 extern PyObject* pmech_types;  // Python map for name to Mechanism
 extern PyObject* rangevars_;   // Python map for name to Symbol
 
+extern "C" int hoc_max_builtin_class_id;
+extern "C" int hoc_return_type_code;
+
 static cTemplate* hoc_vec_template_;
 static cTemplate* hoc_list_template_;
 static cTemplate* hoc_sectionlist_template_;
@@ -86,6 +89,12 @@ static char array_interface_typestr[5] = "|f8";
 // time
 static PyObject* pfunc_get_docstring = NULL;
 
+// Methods unique to the HocTopLevelInterpreter type of HocObject
+// follow the add_methods implementation of python3.6.2 in typeobject.c
+// and the GenericGetAttr implementation in object.c
+static PyObject* topmethdict = NULL;
+static void add2topdict(PyObject*);
+
 static const char* hocobj_docstring =
     "class neuron.hoc.HocObject - Hoc Object wrapper";
 
@@ -366,7 +375,7 @@ static int hocobj_pushargs(PyObject* args, std::vector<char*>& s2free) {
 static void hocobj_pushargs_free_strings(std::vector<char*>& s2free) {
   std::vector<char*>::iterator it = s2free.begin();
   for (; it != s2free.end(); ++it) {
-    delete *it;
+    free(*it);
   }
 }
 
@@ -398,8 +407,11 @@ static Symbol* getsym(char* name, Object* ho, int fail) {
 
 // on entry the stack order is indices, args, object
 // on exit all that is popped and the result is on the stack
-static void component(PyHocObject* po) {
+// returns hoc_return_is_int if called on a builtin (i.e. 2 if bool, 1 if int, 0 otherwise)
+static int component(PyHocObject* po) {
   Inst fc[6];
+  int var_type = 0;
+  hoc_return_type_code = 0;
   fc[0].sym = po->sym_;
   fc[1].i = 0;
   fc[2].i = 0;
@@ -418,6 +430,12 @@ static void component(PyHocObject* po) {
   Inst* pcsav = save_pc(fc);
   hoc_object_component();
   hoc_pc = pcsav;
+  // only return a type if we're calling a built-in (these are all registered first)
+  if (po->ho_->ctemplate->id <= hoc_max_builtin_class_id) {
+    var_type = hoc_return_type_code;
+  }
+  hoc_return_type_code = 0;
+  return(var_type);
 }
 
 int nrnpy_numbercheck(PyObject* po) {
@@ -565,6 +583,15 @@ static int set_final_from_stk(PyObject* po) {
   return err;
 }
 
+
+static void* nrnpy_hoc_int_pop() {
+  return (void*) Py_BuildValue("i", (long) hoc_xpop());
+}
+
+static void* nrnpy_hoc_bool_pop() {
+  return (void*) PyBool_FromLong((long) hoc_xpop());
+}
+
 static void* fcall(void* vself, void* vargs) {
   PyHocObject* self = (PyHocObject*)vself;
   if (self->ho_) {
@@ -572,11 +599,19 @@ static void* fcall(void* vself, void* vargs) {
   }
   std::vector<char*> strings_to_free;
   int narg = hocobj_pushargs((PyObject*)vargs, strings_to_free);
+  int var_type;
   if (self->ho_) {
     self->nindex_ = narg;
-    component(self);
+    var_type = component(self);
     hocobj_pushargs_free_strings(strings_to_free);
-    return (void*)nrnpy_hoc_pop();
+    switch(var_type) {
+      case 2:
+        return nrnpy_hoc_bool_pop();
+      case 1:
+        return nrnpy_hoc_int_pop();
+      default:
+        return(void*) nrnpy_hoc_pop();
+    }
   }
   if (self->sym_->type == BLTIN) {
     if (narg != 1) {
@@ -604,6 +639,7 @@ static void* fcall(void* vself, void* vargs) {
     HocContextRestore
   }
   hocobj_pushargs_free_strings(strings_to_free);
+
   return (void*)nrnpy_hoc_pop();
 }
 
@@ -803,6 +839,22 @@ static int setup_doc_system() {
   return 1;
 }
 
+PyObject* toplevel_get(PyObject* subself, const char* n) {
+  PyHocObject* self = (PyHocObject*)subself;
+  PyObject* result = NULL;
+  if (self->type_ == PyHoc::HocTopLevelInterpreter) {
+    PyObject* descr = PyDict_GetItemString(topmethdict, n);
+    if (descr) {
+      Py_INCREF(descr);
+      descrgetfunc f = descr->ob_type->tp_descr_get;
+      assert(f);
+      result = f(descr, subself, (PyObject*)Py_TYPE(subself));
+      Py_DECREF(descr);
+    }
+  }
+  return result;
+}
+
 // TODO: This function needs refactoring; there are too many exit points
 static PyObject* hocobj_getattr(PyObject* subself, PyObject* pyname) {
   PyHocObject* self = (PyHocObject*)subself;
@@ -828,6 +880,12 @@ static PyObject* hocobj_getattr(PyObject* subself, PyObject* pyname) {
       PyObject* p = nrnpy_hoc2pyobject(self->ho_);
       return PyObject_GenericGetAttr(p, pyname);
     }
+    if (self->type_ == PyHoc::HocTopLevelInterpreter) {
+      result = toplevel_get(subself, n);
+      if (result) {
+        return result;
+      }
+    }
     if (strcmp(n, "__dict__") == 0) {
       // all the public names
       Symlist* sl = 0;
@@ -842,6 +900,7 @@ static PyObject* hocobj_getattr(PyObject* subself, PyObject* pyname) {
       } else {
         symlist2dict(hoc_built_in_symlist, dict);
         symlist2dict(hoc_top_level_symlist, dict);
+        add2topdict(dict);
       }
 
       // Is the self->ho_ a Vector?  If so, add the __array_interface__ symbol
@@ -916,6 +975,20 @@ static PyObject* hocobj_getattr(PyObject* subself, PyObject* pyname) {
         nrn_popsec();
       }
       return result;
+    } else if (self->type_ == PyHoc::HocTopLevelInterpreter &&
+               strncmp(n, "__pysec_", 8) == 0) {
+      Section* sec = (Section*)hoc_pysec_name2ptr(n, 0);
+      if (sec == NULL) {
+        PyErr_SetString(PyExc_NameError, n);
+      } else if (sec && sec->prop && sec->prop->dparam[PROP_PY_INDEX]._pvoid) {
+        result = (PyObject*)sec->prop->dparam[PROP_PY_INDEX]._pvoid;
+        Py_INCREF(result);
+      } else {
+        nrn_pushsec(sec);
+        result = nrnpy_cas(NULL, NULL);
+        nrn_popsec();
+      }
+      return result;
     } else {
       // ipython wants to know if there is a __getitem__
       // even though it does not use it.
@@ -1072,7 +1145,7 @@ static PyObject* hocobj_getattr(PyObject* subself, PyObject* pyname) {
       break;
     }
     case SETPOINTERKEYWORD:
-      result = PyObject_GenericGetAttr((PyObject*)subself, pyname);
+      result = toplevel_get(subself, n);
       break;
     default:  // otherwise
     {
@@ -1358,7 +1431,7 @@ static int hocobj_nonzero(PyObject* self) {
     Arrayinfo* a = hocobj_aray(po->sym_, po->ho_);
     b = araylen(a, po) > 0;
   } else if (po->sym_ && po->sym_->type == TEMPLATE) {
-    b = po->sym_->u.ctemplate->count > 0;
+    b = 1; // prior behavior: po->sym_->u.ctemplate->count > 0;
   }
   return b;
 }
@@ -1777,8 +1850,74 @@ PyObject* nrn_ptr_richcmp(void* self_ptr, void* other_ptr, int op) {
 static PyObject* hocobj_richcmp(PyHocObject* self, PyObject* other, int op) {
   void* self_ptr = (void*)(self->ho_);
   void* other_ptr = (void*)other;
+  bool are_equal = true;
   if (PyObject_TypeCheck(other, hocobject_type)) {
-    other_ptr = (void*)(((PyHocObject*)other)->ho_);
+    if (((PyHocObject*)other)->type_ == self->type_) {
+      switch(self->type_) {
+        case PyHoc::HocRefNum:
+        case PyHoc::HocRefStr:
+        case PyHoc::HocRefObj:
+          /* only same objects can point to same h.ref */
+          self_ptr = (void*) self;
+          break;
+        case PyHoc::HocFunction:
+          if (self->ho_ != (void*)(((PyHocObject*)other)->ho_)) {
+            if (op == Py_NE) {
+              Py_RETURN_TRUE;
+            } else if (op == Py_EQ) {
+              Py_RETURN_FALSE;
+            }
+            /* different classes, comparing < or > doesn't make sense */
+            PyErr_SetString(PyExc_TypeError, "this comparison is undefined");
+            return NULL;
+          }
+          self_ptr = (void*) self->sym_;
+          other_ptr = (void*) (((PyHocObject*)other)->sym_);
+          break;
+        case PyHoc::HocScalarPtr:
+          self_ptr = self->u.px_;
+          other_ptr = (void*)(((PyHocObject*)other)->u.px_);
+          break;
+        case PyHoc::HocArrayIncomplete:
+        case PyHoc::HocArray:
+          if (op != Py_EQ && op != Py_NE) {
+            /* comparing partial arrays doesn't make sense */
+            PyErr_SetString(PyExc_TypeError, "this comparison is undefined");
+            return NULL;
+          }
+          if (self->ho_ != (void*)(((PyHocObject*)other)->ho_)) {
+            /* different objects */
+            other_ptr = (void*)(((PyHocObject*)other)->ho_);
+            break;
+          }
+          if (self->nindex_ != (((PyHocObject*)other)->nindex_) || self->sym_ != (((PyHocObject*)other)->sym_)) {
+            if (op == Py_NE) {
+              Py_RETURN_TRUE;
+            }
+            Py_RETURN_FALSE;
+          }
+          for (int i = 0; i < self->nindex_; i++) {
+            if (self->indices_[i] != ((PyHocObject*)other)->indices_[i]) {
+              are_equal = false;
+            }
+          }
+          if (are_equal == (op == Py_EQ)) {
+            Py_RETURN_TRUE;
+          }
+          Py_RETURN_FALSE;
+        default:
+          other_ptr = (void*)(((PyHocObject*)other)->ho_);
+      }
+    } else {
+      if (op == Py_EQ) {
+        Py_RETURN_FALSE;
+      } else if (op == Py_NE) {
+        Py_RETURN_TRUE;
+      } 
+      /* different NEURON object types are incomperable besides for (in)equality */
+      PyErr_SetString(PyExc_TypeError, "this comparison is undefined");
+      return NULL;
+    }
   }
   return nrn_ptr_richcmp(self_ptr, other_ptr, op);
 }
@@ -2125,18 +2264,10 @@ static PyObject* hocpickle_setstate(PyObject* self, PyObject* args) {
   return Py_None;
 }
 
+// available for every HocObject
 static PyMethodDef hocobj_methods[] = {
-    {"ref", mkref, METH_VARARGS,
-     "Wrap to allow call by reference in a hoc function"},
     {"baseattr", hocobj_baseattr, METH_VARARGS,
      "To allow use of an overrided base method"},
-    {"cas", nrnpy_cas, METH_VARARGS, "Return the currently accessed section."},
-    {"allsec", nrnpy_forall, METH_VARARGS,
-     "Return iterator over all sections."},
-    {"Section", (PyCFunction)nrnpy_newsecobj, METH_VARARGS | METH_KEYWORDS,
-     "Return a new Section"},
-    {"setpointer", setpointer, METH_VARARGS,
-     "Assign hoc variable address to NMODL POINTER"},
     {"hocobjptr", hocobj_vptr, METH_NOARGS, "Hoc Object pointer as a long int"},
     {"same", (PyCFunction)hocobj_same, METH_VARARGS,
      "o1.same(o2) return True if o1 and o2 wrap the same internal HOC Object"},
@@ -2146,6 +2277,34 @@ static PyMethodDef hocobj_methods[] = {
     {"__setstate__", hocpickle_setstate, METH_VARARGS, "pickle interface"},
     {NULL, NULL, 0, NULL}};
 
+// only for a HocTopLevelInterpreter type HocObject
+static PyMethodDef toplevel_methods[] = {
+    {"ref", mkref, METH_VARARGS,
+     "Wrap to allow call by reference in a hoc function"},
+    {"cas", nrnpy_cas, METH_VARARGS, "Return the currently accessed section."},
+    {"allsec", nrnpy_forall, METH_VARARGS,
+     "Return iterator over all sections."},
+    {"Section", (PyCFunction)nrnpy_newsecobj, METH_VARARGS | METH_KEYWORDS,
+     "Return a new Section"},
+    {"setpointer", setpointer, METH_VARARGS,
+     "Assign hoc variable address to NMODL POINTER"},
+    {NULL, NULL, 0, NULL}};
+
+static void add2topdict(PyObject* dict) {
+  for (PyMethodDef* meth = toplevel_methods; meth->ml_name != NULL; meth++) {
+    int err;
+    PyObject* nn = Py_BuildValue("s", meth->ml_doc);
+    if (!nn) {
+      return;
+    }
+    err = PyDict_SetItemString(dict, meth->ml_name, nn);
+    Py_DECREF(nn);
+    if (err) {
+      return;
+    }
+  }
+}
+
 static PyMemberDef hocobj_members[] = {{NULL, 0, 0, 0, NULL}};
 
 #if (PY_MAJOR_VERSION >= 3)
@@ -2228,6 +2387,19 @@ myPyMODINIT_FUNC nrnpy_hoc() {
   // printf("AddObject HocObject\n");
   PyModule_AddObject(m, "HocObject", (PyObject*)hocobject_type);
 
+  topmethdict = PyDict_New();
+  for (PyMethodDef* meth = toplevel_methods; meth->ml_name != NULL; meth++) {
+    PyObject* descr;
+    int err;
+    descr = PyDescr_NewMethod(hocobject_type, meth);
+    assert(descr);
+    err = PyDict_SetItemString(topmethdict, meth->ml_name, descr);
+    Py_DECREF(descr);
+    if (err < 0) {
+      goto fail;
+    }      
+  }
+
   s = hoc_lookup("Vector");
   assert(s);
   hoc_vec_template_ = s->u.ctemplate;
