@@ -3,10 +3,22 @@
 #include <InterViews/resource.h>
 #include <nrnoc2iv.h>
 #include "nrnpy_utils.h"
+#include <cmath>
+#ifndef M_PI
+#define M_PI (3.14159265358979323846)
+#endif
 
 extern "C" {
 #include <membfunc.h>
 #include <parse.h>
+extern void nrn_pt3dremove(Section* sec, int i0);
+extern void nrn_pt3dinsert(Section* sec, int i0, double x, double y, double z, double d);
+extern void nrn_pt3dclear(Section* sec, int req);
+extern void nrn_pt3dchange1(Section* sec, int i, double d);
+extern void nrn_pt3dchange2(Section* sec, int i, double x, double y, double z, double diam);
+extern void stor_pt3d(Section* sec, double x, double y, double z, double d);
+extern void nrn_pt3dstyle1(Section* sec, double x, double y, double z);
+extern void nrn_pt3dstyle0(Section* sec);
 extern void nrn_area_ri(Section* sec);
 extern void sec_free(hoc_Item*);
 extern Symlist* hoc_built_in_symlist;
@@ -46,6 +58,7 @@ typedef struct {
   PyObject_HEAD NPySegObj* pyseg_;
   Symbol* sym_;
   int isptr_;
+  int attr_from_sec_; // so section.xraxial[0] = e assigns to all segments.
 } NPyRangeVar;
 
 static PyTypeObject* psection_type;
@@ -101,6 +114,7 @@ static char* pysec_name(Section* sec) {
   if (sec->prop) {
     NPySecObj* ps = (NPySecObj*)sec->prop->dparam[PROP_PY_INDEX]._pvoid;
     buf[0] = '\0';
+#if 0 // full name in ps->name because of nrnpy_pysecname2sec_remove
     if (ps->cell_) {
       PyLockGIL lock;
 
@@ -110,6 +124,7 @@ static char* pysec_name(Section* sec) {
       char* cp = str.c_str();
       sprintf(buf, "%s.", cp);
     }
+#endif
     char* cp = buf + strlen(buf);
     if (ps->name_) {
       sprintf(cp, "%s", ps->name_);
@@ -141,12 +156,13 @@ static int pysec_cell_equals(Section* sec, Object* obj) {
 static void NPySecObj_dealloc(NPySecObj* self) {
   // printf("NPySecObj_dealloc %p %s\n", self, secname(self->sec_));
   if (self->sec_) {
+    if (self->name_) {
+      nrnpy_pysecname2sec_remove(self->sec_);
+      delete[] self->name_;
+    }
     if (self->sec_->prop) {
       self->sec_->prop->dparam[PROP_PY_INDEX]._pvoid = 0;
     }
-    if (self->name_) {
-      delete[] self->name_;
-    }
     if (self->sec_->prop && !self->sec_->prop->dparam[0].sym) {
       sec_free(self->sec_->prop->dparam[8].itm);
     } else {
@@ -209,10 +225,30 @@ static int NPySecObj_init(NPySecObj* self, PyObject* args, PyObject* kwds) {
     }
     // note that we are NOT referencing the cell
     if (name) {
-      self->name_ = new char[strlen(name) + 1];
-      strcpy(self->name_, name);
+      size_t n = strlen(name) + 1; // include terminator
+#if 1
+      if (self->cell_) {
+        // include cellname in name so nrnpy_pysecname2sec_remove can determine
+
+        PyObject* cell = PyObject_Str(self->cell_);
+        if (cell == NULL) {
+          return -1;
+        }
+        Py2NRNString str(cell);
+        Py_DECREF(cell);
+        char* cp = str.c_str();
+        n += strlen(cp) + 1; // include dot
+        self->name_ = new char[n];
+        sprintf(self->name_, "%s.%s", cp, name);
+      }else
+#endif
+      {
+        self->name_ = new char[n];
+        strcpy(self->name_, name);
+      }
     }
     self->sec_ = nrnpy_newsection(self);
+    nrnpy_pysecname2sec_add(self->sec_);
   }
   return 0;
 }
@@ -313,6 +349,7 @@ static PyObject* NPyRangeVar_new(PyTypeObject* type, PyObject* args,
     self->pyseg_ = NULL;
     self->sym_ = NULL;
     self->isptr_ = 0;
+    self->attr_from_sec_ = 0;
   }
   return (PyObject*)self;
 }
@@ -391,6 +428,132 @@ static PyObject* NPySecObj_n3d(NPySecObj* self) {
   return PyInt_FromLong(self->sec_->npt3d);
 }
 
+static PyObject* NPySecObj_pt3dremove(NPySecObj* self, PyObject* args) {
+  Section* sec = self->sec_;
+  int i0, n;
+  if (!PyArg_ParseTuple(args, "i", &i0)) {
+    return NULL;
+  }
+  if (i0 < 0 || i0 >= sec->npt3d) {
+    PyErr_SetString(PyExc_Exception, "Arg out of range\n");
+    return NULL;
+  }
+  
+  nrn_pt3dremove(sec, i0);
+  Py_RETURN_NONE;
+}
+
+static PyObject* NPySecObj_pt3dclear(NPySecObj* self, PyObject* args) {
+  Section* sec = self->sec_;
+  int req = 0;
+  Py_ssize_t narg = PyTuple_GET_SIZE(args);
+  if (narg) {
+    if (!PyArg_ParseTuple(args, "i", &req)) {
+      return NULL;
+    }
+  }
+  if (req < 0) {
+    PyErr_SetString(PyExc_Exception, "Arg out of range\n");
+    return NULL;
+  }
+  nrn_pt3dclear(sec, req);
+  return PyInt_FromLong(sec->pt3d_bsize);
+}
+
+static PyObject* NPySecObj_pt3dchange(NPySecObj* self, PyObject* args) {
+  Section* sec = self->sec_;
+  int i;
+  double x, y, z, diam;
+  Py_ssize_t narg = PyTuple_GET_SIZE(args);
+  if (narg == 2) {
+    if (!PyArg_ParseTuple(args, "id", &i, &diam)) {
+      return NULL;
+    }
+    if (i < 0 || i >= sec->npt3d) {
+      PyErr_SetString(PyExc_Exception, "Arg out of range\n");
+      return NULL;
+    }
+    nrn_pt3dchange1(sec, i, diam);
+  }
+  else if (narg == 5) {
+    if (!PyArg_ParseTuple(args, "idddd", &i, &x, &y, &z, &diam)) {
+      return NULL;
+    }
+    if (i < 0 || i >= sec->npt3d) {
+      PyErr_SetString(PyExc_Exception, "Arg out of range\n");
+      return NULL;
+    }
+    nrn_pt3dchange2(sec, i, x, y, z, diam);
+  } else {
+    PyErr_SetString(PyExc_Exception, "Wrong number of arguments\n");
+    return NULL;
+  }
+  Py_RETURN_NONE;
+}
+
+static PyObject* NPySecObj_pt3dinsert(NPySecObj* self, PyObject* args) {
+  Section* sec = self->sec_;
+  int i;
+  double x, y, z, d;
+  if (!PyArg_ParseTuple(args, "idddd", &i, &x, &y, &z, &d)) {
+    return NULL;
+  }
+  if (i < 0 || i > sec->npt3d) {
+    PyErr_SetString(PyExc_Exception, "Arg out of range\n");
+    return NULL;
+  }
+  nrn_pt3dinsert(sec, i, x, y, z, d);
+  Py_RETURN_NONE;
+}
+
+
+static PyObject* NPySecObj_pt3dadd(NPySecObj* self, PyObject* args) {
+  Section* sec = self->sec_;
+  double x, y, z, d;
+  // TODO: add support for iterables
+  if (!PyArg_ParseTuple(args, "dddd", &x, &y, &z, &d)) {
+    return NULL;
+  }
+  stor_pt3d(sec, x, y, z, d);
+  Py_RETURN_NONE;
+}
+
+static PyObject* NPySecObj_pt3dstyle(NPySecObj* self, PyObject* args) {
+  Section* sec = self->sec_;
+  int style;
+  double x, y, z;
+  Py_ssize_t narg = PyTuple_GET_SIZE(args);
+
+  if (narg) {
+    if (narg == 1) {
+      if (!PyArg_ParseTuple(args, "i", &style)) {
+        return NULL;
+      }
+      if (style) {
+        PyErr_SetString(PyExc_AttributeError, "If exactly one argument, it must be 0.");
+        return NULL;
+      }
+      nrn_pt3dstyle0(sec);
+    } else if (narg == 4) {
+      // TODO: figure out some way of reading the logical connection point
+      // don't use hoc refs
+      if (!PyArg_ParseTuple(args, "iddd", &style, &x, &y, &z)) {
+        return NULL;
+      }
+      nrn_pt3dstyle1(sec, x, y, z);
+
+    } else {
+      PyErr_SetString(PyExc_Exception, "Wrong number of arguments.");
+      return NULL;
+    }
+  }
+
+  if (sec->logical_connection) {
+    Py_RETURN_TRUE;
+  }
+  Py_RETURN_FALSE;
+}
+
 static PyObject* NPySecObj_x3d(
     NPySecObj* self, PyObject* args) {  // returns x value at index of 3d list
   Section* sec = self->sec_;
@@ -468,6 +631,25 @@ static PyObject* NPySecObj_diam3d(
   return PyFloat_FromDouble((double)fabs(sec->pt3d[i].d));
 }
 
+static PyObject* NPySecObj_spine3d(
+    NPySecObj* self,
+    PyObject* args) {  // returns True/False depending on if spine present
+  Section* sec = self->sec_;
+  int n, i;
+  if (!PyArg_ParseTuple(args, "i", &i)) {
+    return NULL;
+  }
+  n = sec->npt3d - 1;
+  if (i < 0 || i > n) {
+    PyErr_SetString(PyExc_Exception, "Arg out of range\n");
+    return NULL;
+  }
+  if (sec->pt3d[i].d < 0) {
+    Py_RETURN_TRUE;
+  }
+  Py_RETURN_FALSE;
+}
+
 static PyObject* pysec_repr(PyObject* p) {
   NPySecObj* psec = (NPySecObj*)p;
   if (psec->sec_ && psec->sec_->prop) {
@@ -497,6 +679,43 @@ static PyObject* hoc_internal_name(NPySecObj* self) {
   return result;
 }
 
+static PyObject* nrnpy_psection;
+static PyObject* nrnpy_set_psection(PyObject* self, PyObject* args) {
+  PyObject* po;
+  if (!PyArg_ParseTuple(args, "O", &po)) {
+    return NULL;
+  }
+  if (PyCallable_Check(po) == 0) {
+    PyErr_SetString(PyExc_TypeError, "argument must be a callable");
+    return NULL;
+  }
+  if (nrnpy_psection) {
+    Py_DECREF(nrnpy_psection);
+    nrnpy_psection = NULL;
+  }
+  nrnpy_psection = po;
+  Py_INCREF(po);
+  return po;
+}
+
+static PyObject* NPySecObj_psection(NPySecObj* self) {
+  if (nrnpy_psection) {
+    PyObject* arglist = Py_BuildValue("(O)", self);
+    PyObject* result = PyEval_CallObject(nrnpy_psection, arglist);
+    Py_DECREF(arglist);
+    return result;
+  }
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
+static PyObject* is_pysec(NPySecObj* self) {
+  if (self->sec_->prop && self->sec_->prop->dparam[PROP_PY_INDEX]._pvoid) {
+      Py_RETURN_TRUE;
+  }
+  Py_RETURN_FALSE;
+}
+
 NPySecObj* newpysechelp(Section* sec) {
   if (!sec || !sec->prop) {
     return NULL;
@@ -881,6 +1100,91 @@ static PyObject* seg_area(NPySegObj* self) {
   return result;
 }
 
+static inline double scaled_frustum_volume(double length, double d0, double d1) {
+  // this is proportional to the volume of a frustum... need to multiply by pi/12
+  return length * (d0 * d0 + d0 * d1 + d1 * d1);
+}
+
+static inline double interpolate(double x0, double x1, double y0, double y1, double xnew) {
+  // computes ynew on the line between (x0, y0) and (x1, y1)
+  // recall: y - y0 = m (x - x0)
+  if (x1 == x0) {
+    // avoid dividing by 0 if no length... 
+    return y0;
+  } else {
+    return y0 + (y1 - y0) * (xnew - x0) / (x1 - x0);
+  }
+}
+
+static int arg_bisect_arc3d(Section* sec, int npt3d, double x) {
+  // returns the index in sec where the arc is no more than x, but where the next points arc is more
+  // right endpoint is never included
+  int left = 0;
+  int right = npt3d;
+  while (right - left > 1) {
+    int mid = (left + right) / 2;
+    if (sec->pt3d[mid].arc < x) {
+      left = mid;
+    } else {
+      right = mid;
+    }
+  }
+  return left;
+}
+
+static PyObject* seg_volume(NPySegObj* self) {
+  Section* sec = self->pysec_->sec_;
+  int i;
+  if (sec->recalc_area_) {
+    nrn_area_ri(sec);
+  }
+  double x = self->x_;
+  double a = 0.0;
+  // volume only exists on the interior
+  if (x > 0. && x < 1.) {
+    // every section owns one extra node (the conservation node)
+    int nseg = sec->nnode - 1;
+    double length = section_length(sec) / nseg;
+    int iseg = x * nseg;
+    double seg_left_arc = iseg * length;
+    double seg_right_arc = (iseg + 1) * length;
+    if (sec->npt3d > 1) {
+      int i_left = arg_bisect_arc3d(sec, sec->npt3d, seg_left_arc);
+      double left_diam = fabs(sec->pt3d[i_left].d);
+      double right_diam = fabs(sec->pt3d[i_left + 1].d);
+      double left_arc = seg_left_arc;
+      left_diam = interpolate(sec->pt3d[i_left].arc, sec->pt3d[i_left + 1].arc, left_diam, right_diam, left_arc);
+
+      for (i = i_left + 1; i < sec->npt3d && sec->pt3d[i].arc < seg_right_arc; i++) {
+        right_diam = fabs(sec->pt3d[i].d);
+        a += scaled_frustum_volume(sec->pt3d[i].arc - left_arc, left_diam, right_diam);
+        left_arc = sec->pt3d[i].arc;
+        left_diam = right_diam;
+      }
+      if (i < sec->npt3d) {
+        right_diam = interpolate(left_arc, sec->pt3d[i].arc, left_diam, fabs(sec->pt3d[i].d), seg_right_arc);
+        a += scaled_frustum_volume(seg_right_arc - left_arc, left_diam, right_diam);
+      }
+
+      // the scaled_frustum_volume formula factored out a pi/12
+      a *= M_PI / 12;
+    } else {
+      // 0 or 1 3D points... so give cylinder volume
+      Node* nd = node_exact(sec, x);
+      for (Prop* p = nd->prop; p; p = p->next) {
+        if (p->type == MORPHOLOGY) {
+          double diam = p->param[0];
+          a = M_PI * diam * diam / 4 * length;
+          break;
+        }
+      }
+    }
+  }
+  PyObject* result = Py_BuildValue("d", a);
+  return result;
+}
+
+
 static PyObject* seg_ri(NPySegObj* self) {
   Section* sec = self->pysec_->sec_;
   if (sec->recalc_area_) {
@@ -967,7 +1271,7 @@ static void rv_noexist(Section* sec, const char* n, double x, int err) {
   } else {
     sprintf(buf, "%s does not exist at %s(%g)", n, secname(sec), x);
   }
-  PyErr_SetString(PyExc_NameError, buf);
+  PyErr_SetString(PyExc_AttributeError, buf);
 }
 
 static PyObject* section_getattro(NPySecObj* self, PyObject* pyname) {
@@ -998,6 +1302,7 @@ static PyObject* section_getattro(NPySecObj* self, PyObject* pyname) {
       r->pyseg_->x_ = 0.5;
       r->sym_ = sym;
       r->isptr_ = 0;
+      r->attr_from_sec_ = 1;
       result = (PyObject*)r;
     } else {
       int err;
@@ -1225,6 +1530,7 @@ static PyObject* segment_getattro(NPySegObj* self, PyObject* pyname) {
       Py_INCREF(r->pyseg_);
       r->sym_ = sym;
       r->isptr_ = 0;
+      r->attr_from_sec_ = 0;
       result = (PyObject*)r;
     } else {
       int err;
@@ -1251,6 +1557,7 @@ static PyObject* segment_getattro(NPySegObj* self, PyObject* pyname) {
         Py_INCREF(r->pyseg_);
         r->sym_ = sym;
         r->isptr_ = 1;
+        r->attr_from_sec_ = 0;
         result = (PyObject*)r;
       } else {
         int err;
@@ -1353,6 +1660,21 @@ static int segment_setattro(NPySegObj* self, PyObject* pyname,
   return err;
 }
 
+static bool striptrail(char* buf, int sz, const char* n, const char* m) {
+    int nlen = strlen(n);
+    int mlen = strlen(m);
+    int u = nlen - mlen - 1; // should be location of _
+    if (u > 0 && n[u] == '_') { // likely n is name_m
+      if (strcmp(n+(u+1), m) != 0) {
+          return false;
+      }
+      strncpy(buf, n, sz);
+      buf[u] = '\0';
+      return true;
+    }
+    return false;
+}
+
 static PyObject* mech_getattro(NPyMechObj* self, PyObject* pyname) {
   Py_INCREF(pyname);
   Py2NRNString name(pyname);
@@ -1365,8 +1687,9 @@ static PyObject* mech_getattro(NPyMechObj* self, PyObject* pyname) {
   // printf("mech_getattro %s\n", n);
   PyObject* result = NULL;
   NrnProperty np(self->prop_);
-  char buf[200];
   int isptr = (strncmp(n, "_ref_", 5) == 0);
+  int bufsz = strlen(n) + 6;
+  char *buf = new char[bufsz];
   sprintf(buf, "%s_%s", isptr ? n + 5 : n,
           memb_func[self->prop_->type].sym->name);
   Symbol* sym = np.find(buf);
@@ -1378,6 +1701,7 @@ static PyObject* mech_getattro(NPyMechObj* self, PyObject* pyname) {
       Py_INCREF(r->pyseg_);
       r->sym_ = sym;
       r->isptr_ = isptr;
+      r->attr_from_sec_ = 0;
       result = (PyObject*)r;
     } else {
       double* px = np.prop_pval(sym, 0);
@@ -1391,14 +1715,20 @@ static PyObject* mech_getattro(NPyMechObj* self, PyObject* pyname) {
     }
   } else if (strcmp(n, "__dict__") == 0) {
     result = PyDict_New();
+    char* mname = memb_func[self->prop_->type].sym->name;
+    int mnamelen = strlen(mname);
     for (Symbol* s = np.first_var(); np.more_var(); s = np.next_var()) {
-      int err = PyDict_SetItemString(result, s->name, Py_None);
+      if (!striptrail(buf, bufsz, s->name, mname)) {
+        strcpy(buf, s->name);
+      }
+      int err = PyDict_SetItemString(result, buf, Py_None);
       assert(err == 0);
     }
   } else {
     result = PyObject_GenericGetAttr((PyObject*)self, pyname);
   }
   Py_DECREF(pyname);
+  delete [] buf;
   return result;
 }
 
@@ -1508,10 +1838,22 @@ static int rv_setitem(PyObject* self, Py_ssize_t ix, PyObject* value) {
     rv_noexist(r->pyseg_->pysec_->sec_, r->sym_->name, r->pyseg_->x_, err);
     return -1;
   }
-  d += ix;
-  if (!PyArg_Parse(value, "d", d)) {
-    PyErr_SetString(PyExc_ValueError, "bad value");
-    return -1;
+  if (r->attr_from_sec_) {
+    // the range variable array is from a section not a segment and so
+    // assignment is over the entire section.
+    double x;
+    if (!PyArg_Parse(value, "d", &x)) {
+      PyErr_SetString(PyExc_ValueError, "bad value");
+      return -1;
+    }
+    hoc_pushx(double(ix));
+    nrn_rangeconst(r->pyseg_->pysec_->sec_, r->sym_, &x, 0);
+  }else{
+    d += ix;
+    if (!PyArg_Parse(value, "d", d)) {
+      PyErr_SetString(PyExc_ValueError, "bad value");
+      return -1;
+    }
   }
   if (r->sym_->u.rng.type == EXTRACELL && r->sym_->u.rng.index == 0) {
     diam_changed = 1;
@@ -1569,6 +1911,24 @@ static PyMethodDef NPySecObj_methods[] = {
      "Returns the arc position of the ith 3D point."},
     {"diam3d", (PyCFunction)NPySecObj_diam3d, METH_VARARGS,
      "Returns the diam of the ith 3D point."},
+    {"spine3d", (PyCFunction)NPySecObj_spine3d, METH_VARARGS,
+     "Returns True or False depending on whether a spine exists at the ith 3D point."},
+    {"pt3dremove", (PyCFunction) NPySecObj_pt3dremove, METH_VARARGS,
+     "Removes the ith 3D point."},
+    {"pt3dclear", (PyCFunction) NPySecObj_pt3dclear, METH_VARARGS,
+     "Clears all 3D points. Optionally takes a buffer size."},
+    {"pt3dinsert", (PyCFunction) NPySecObj_pt3dinsert, METH_VARARGS,
+     "Insert the point (so it becomes the i'th point) to section. If i is equal to sec.n3d(), the point is appended (equivalent to sec.pt3dadd())"},
+    {"pt3dchange", (PyCFunction) NPySecObj_pt3dchange, METH_VARARGS,
+     "Change the i'th 3-d point info. If only two args then the second arg is the diameter and the location is unchanged."},
+    {"pt3dadd", (PyCFunction) NPySecObj_pt3dadd, METH_VARARGS,
+     "Add the 3d location and diameter point (or points in the second form) at the end of the current pt3d list. Assume that successive additions increase the arc length monotonically."},
+    {"pt3dstyle", (PyCFunction) NPySecObj_pt3dstyle, METH_VARARGS,
+     "Returns True if using a logical connection point, else False. With first arg 0 sets to no logical connection point. With first arg 1 and x, y, z arguments, sets the logical connection point. Return value includes the result of any setting."},  
+    {"is_pysec", (PyCFunction)is_pysec, METH_NOARGS,
+     "Returns True if Section created from Python, False if created from HOC."},
+    {"psection", (PyCFunction)NPySecObj_psection, METH_NOARGS,
+     "Returns dict of info about Section contents."},
     {NULL}};
 
 static PyMethodDef NPySegObj_methods[] = {
@@ -1583,6 +1943,8 @@ static PyMethodDef NPySegObj_methods[] = {
     {"ri", (PyCFunction)seg_ri, METH_NOARGS,
      "Segment resistance to parent segment (Megohms) (same as h.ri(sec(x), "
      "sec=sec))"},
+    {"volume", (PyCFunction)seg_volume, METH_NOARGS,
+     "Segment volume (um3)"},
     {NULL}};
 
 // I'm guessing Python should change their typedef to get rid of the
@@ -1624,6 +1986,8 @@ static PyMethodDef nrnpy_methods[] = {
     {"cas", nrnpy_cas, METH_VARARGS, "Return the currently accessed section."},
     {"allsec", nrnpy_forall, METH_VARARGS,
      "Return iterator over all sections."},
+    {"set_psection", nrnpy_set_psection, METH_VARARGS,
+     "Specify the nrn.Section.psection callback."},
     {NULL}};
 
 #if PY_MAJOR_VERSION >= 3
@@ -1640,6 +2004,7 @@ static void rangevars_add(Symbol* sym) {
   // printf("%s\n", sym->name);
   r->sym_ = sym;
   r->isptr_ = 0;
+  r->attr_from_sec_ = 0;
   PyDict_SetItemString(rangevars_, sym->name, (PyObject*)r);
 }
 
@@ -1772,6 +2137,7 @@ void nrnpy_reg_mech(int type) {
       Symbol* s = hoc_table_lookup("get_segment", sl);
       if (!s) {
         s = hoc_install("get_segment", OBFUNCTION, 0, &sl);
+        s->cpublic = 1;
 #if MAC
         s->u.u_proc->defn.pfo = (Object * *(*)(...))pp_get_segment;
 #else
