@@ -568,8 +568,13 @@ static double func_call(Object* ho, int narg, int* err) {
 static double guigetval(Object* ho) {
   PyObject* po = ((Py2Nrn*)ho->u.this_pointer)->po_;
   PyLockGIL lock;
-  PyObject* r = PyObject_GetAttr(PyTuple_GetItem(po, 0),
-                                 PyTuple_GetItem(po, 1));
+  PyObject* r = NULL;
+  PyObject* p = PyTuple_GetItem(po, 0);
+  if (PySequence_Check(p) || PyMapping_Check(p)) {
+    r = PyObject_GetItem(p, PyTuple_GetItem(po, 1));
+  }else{
+    r = PyObject_GetAttr(p, PyTuple_GetItem(po, 1));
+  }
   PyObject* pn = PyNumber_Float(r);
   double x = PyFloat_AsDouble(pn);
   Py_XDECREF(pn);
@@ -580,8 +585,11 @@ static void guisetval(Object* ho, double x) {
   PyObject* po = ((Py2Nrn*)ho->u.this_pointer)->po_;
   PyLockGIL lock;
   PyObject* pn = PyFloat_FromDouble(x);
-  if (PyObject_SetAttr(PyTuple_GetItem(po, 0), PyTuple_GetItem(po, 1), pn)) {
-    ;
+  PyObject* p = PyTuple_GetItem(po, 0);
+  if (PySequence_Check(p) || PyMapping_Check(p)) {
+    PyObject_SetItem(p, PyTuple_GetItem(po, 1), pn);
+  }else{
+    PyObject_SetAttr(p, PyTuple_GetItem(po, 1), pn);
   }
   Py_XDECREF(pn);
 }
@@ -853,5 +861,7 @@ Object* py_alltoall(Object* o, int size) {
   Object* ho = nrnpy_po2ho(pdest);
   --ho->refcount;
   return ho;
+#else
+  return o;
 #endif
 }
