@@ -38,10 +38,15 @@ int (*p_nrnpy_pyrun)(const char* fname);
 
 #if carbon || defined(MINGW)
 #include <pthread.h>
+extern int stdin_event_ready();
 #endif
 
 #if HAVE_FENV_H
-#define NRN_FLOAT_EXCEPTION (defined(linux))
+#if defined(linux)
+#define NRN_FLOAT_EXCEPTION 1
+#else
+#define NRN_FLOAT_EXCEPTION 0
+#endif
 #endif
 
 #if NRN_FLOAT_EXCEPTION
@@ -189,10 +194,13 @@ int	gargc;
 static int c = '\n';	/* global for use by warning() */
 
 #if defined(WIN32) || MAC
-set_intset() {
+void set_intset() {
 	intset++;
 }
 #endif
+#ifdef WIN32
+extern void hoc_win32_cleanup();
+#endif
 
 static int follow(int expect, int ifyes, int ifno);/* look ahead for >=, etc. */
 static int Getc(NrnFILEWrap* fp);
@@ -274,6 +282,27 @@ static int lexstate = 0;
    at eos to see if true.*/
 static int eos;
 
+static char* optarray(char* buf) {
+  int c;
+  if ((c = Getc(fin)) == '[') {
+    char* s = buf + strlen(buf);
+    *s++ = c;
+    while (isdigit(c = Getc(fin)) && (s - buf) < 200) {
+      *s++ = c;
+    }
+    if (c == ']') {
+      *s++ = c;
+      *s = '\0';
+    }else{
+      *s = '\0';
+      acterror(buf, " not literal name[int]");
+    }
+  }else{
+    unGetc(c, fin);
+  }
+  return buf;
+}
+
 int yylex(void)			/* hoc6 */
 {
    restart:	/* when no token in between comments */
@@ -356,6 +385,38 @@ int yylex(void)			/* hoc6 */
 			yylval.ptr = hoc_sec_internal_name2ptr(sbuf, 1);
 			return INTERNALSECTIONNAME;
 		}
+		/* _pysec.name[int] or _pysec.name[int].name[int] where
+		  [int] is optional must resolve to Section at parse time.
+		  void* nrn_parsing_pysec_ is 1 to signal the beginning
+		  of parsing and as a sub-dictionary pointer in case the
+		  first level __psec.name[int] is the name of a cell.
+		  On error or success in parse.y, nrn_parsing_pysec_ is
+		  set back to NULL.
+		*/
+		if (strcmp(sbuf, "_pysec") == 0) {
+			if (c != '.') {
+hoc_acterror("Must be of form _pysec.secname where secname is the literal result of nrn.Section.name() .", NULL);
+			}
+			yylval.ptr = NULL;
+			nrn_parsing_pysec_ = (void*)1;
+			return PYSEC;
+		}
+		if (nrn_parsing_pysec_) {
+			yylval.ptr = hoc_pysec_name2ptr(optarray(sbuf), 1);
+			if (nrn_parsing_pysec_ > (void*)1) { /* there will be a second part */
+				c = Getc(fin); unGetc(c, fin);
+				if (c != '.') { /* so there must be a . */
+					nrn_parsing_pysec_ = NULL;
+hoc_acterror("Must be of form _pysec.cellname.secname where cellname.secname is the literal result of nrn.Section.name() .", NULL);
+				}
+			}
+			if (yylval.ptr == NULL) {
+				return PYSECOBJ;
+			}else{
+				nrn_parsing_pysec_ = NULL;
+				return PYSECNAME;
+			}
+		}
 		if ((s=lookup(sbuf)) == 0)
 			s = install(sbuf, UNDEF, 0.0, &symlist);
 		yylval.sym = s;
@@ -823,6 +884,7 @@ static void hoc_run1(void);
 int hoc_main1(int argc, const char** argv, const char** envp)	/* hoc6 */
 {
 #ifdef WIN32
+	extern void hoc_set_unhandled_exception_filter();
 	hoc_set_unhandled_exception_filter();
 #endif
 #if 0
@@ -916,11 +978,9 @@ static pthread_mutex_t inputMutex_;
 static pthread_cond_t inputCond_;
 static int inputReadyFlag_;
 
-void* inputReadyThread(void*  input);
 void* inputReadyThread(void* input) {
 	fd_set readfds;
 	int i, j;
-	extern int stdin_event_ready();
 	char c;
 //	printf("inputReadyThread started\n");
 //	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, &j);
@@ -951,11 +1011,10 @@ static pthread_mutex_t inputMutex_;
 static pthread_cond_t inputCond_;
 static int inputReadyFlag_;
 static int inputReadyVal_;
+extern int getch();
 
-void* inputReadyThread(void*  input);
 void* inputReadyThread(void* input) {
 	int i, j;
-	extern int stdin_event_ready();
 	char c;
 //	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, &j);
 	for (;;) {
@@ -1425,11 +1484,7 @@ void warning(const char* s, const char* t)	/* print warning message */
 	Fprintf(stderr, "%s %s",id, cbuf);
     if (nrnmpi_numprocs_world > 0) {
 	for (cp = cbuf; cp != ctp; cp++) {
-#if defined(WIN32) && !defined(CYGWIN)
-		fputchar(' ');
-#else
-		IGNORE(putc(' ', stderr));
-#endif
+		Fprintf(stderr, " ");
 	}
 	Fprintf(stderr,"^\n");
     }
