@@ -17,6 +17,8 @@
 #undef MD
 #define MD 2147483647.
 
+extern "C" int hoc_return_type_code;
+
 extern "C" {
 	extern int vector_arg_px(int, double**);
 	Symbol* hoc_which_template(Symbol*);
@@ -69,6 +71,7 @@ extern "C" {
 	extern int nrn_how_many_processors();
 	extern size_t nrnbbcore_write();
 	extern size_t nrnbbcore_register_mapping();
+	extern int nrncore_run(const char*);
 
 }
 
@@ -204,6 +207,7 @@ static double working(void* v) {
 	OcBBS* bbs = (OcBBS*)v;
 	int id;
 	bool b = bbs->working(id, bbs->retval_, bbs->userid_);
+	hoc_return_type_code = 1; // integer
 	if (b) {
 		return double(id);
 	}else{
@@ -222,26 +226,32 @@ static double userid(void* v) {
 }
 
 static double nhost(void* v) {
+	hoc_return_type_code = 1;
 	return nrnmpi_numprocs;
 }
 
 static double nrn_rank(void* v) {
+	hoc_return_type_code = 1;
 	return nrnmpi_myid;
 }
 
 static double nhost_world(void* v) {
+	hoc_return_type_code = 1; // integer
 	return nrnmpi_numprocs_world;
 }
 
 static double rank_world(void* v) {
+	hoc_return_type_code = 1;
 	return nrnmpi_myid_world;
 }
 
 static double nhost_bbs(void* v) {
+	hoc_return_type_code = 1;
 	return nrnmpi_numprocs_bbs;
 }
 
 static double rank_bbs(void* v) {
+	hoc_return_type_code = 1;
 	return nrnmpi_myid_bbs;
 }
 
@@ -426,6 +436,7 @@ static double take(void* v) {
 
 static double look(void* v) {
 	OcBBS* bbs = (OcBBS*)v;
+	hoc_return_type_code = 2; // boolean
 	if (bbs->look(key_help())) {
 		unpack_help(2, bbs);
 		return 1.;
@@ -435,6 +446,7 @@ static double look(void* v) {
 
 static double look_take(void* v) {
 	OcBBS* bbs = (OcBBS*)v;
+	hoc_return_type_code = 2; // boolean
 	if (bbs->look_take(key_help())) {
 		unpack_help(2, bbs);
 		return 1.;
@@ -533,6 +545,7 @@ static double set_gid2node(void* v) {
 
 static double gid_exists(void* v) {
 	OcBBS* bbs = (OcBBS*)v;
+	hoc_return_type_code = 1; // NOTE: possible returns are integers 0 - 3
 	return int(bbs->gid_exists(int(chkarg(1, 0, MD))));
 }
 
@@ -615,10 +628,15 @@ static double outputcell(void* v) {
 
 static double spike_record(void* v) {
 	OcBBS* bbs = (OcBBS*)v;
-	int gid = int(chkarg(1, -1., MD));
 	IvocVect* spikevec = vector_arg(2);
 	IvocVect* gidvec = vector_arg(3);	
-	bbs->spike_record(gid, spikevec, gidvec);
+	if (hoc_is_object_arg(1) && is_vector_arg(1)) {
+		IvocVect* gids = vector_arg(1);
+		bbs->spike_record(gids, spikevec, gidvec);
+	}else{
+		int gid = int(chkarg(1, -1., MD));
+		bbs->spike_record(gid, spikevec, gidvec);
+	}
 	return 0.;
 }
 
@@ -636,6 +654,7 @@ static double set_maxstep(void* v) {
 static double spike_stat(void* v) {
 	OcBBS* bbs = (OcBBS*)v;
 	int nsend, nsendmax, nrecv, nrecv_useful;
+	hoc_return_type_code = 1; // integer
 	nsend = nsendmax = nrecv = nrecv_useful = 0;
 	bbs->netpar_spanning_statistics(&nsend, &nsendmax, &nrecv, &nrecv_useful);
 	if (ifarg(1)) { *hoc_pgetarg(1) = nsend; }
@@ -850,6 +869,7 @@ static double checkpoint(void*) {
 
 static double nthrd(void*) {
 	int ip = 1;
+	hoc_return_type_code = 1; // integer
 	if (ifarg(1)) {
 		if (ifarg(2)) { ip = int(chkarg(2, 0, 1)); }
 		nrn_threads_create(int(chkarg(1, 1, 1e5)), ip);
@@ -888,11 +908,13 @@ static double thread_busywait(void*) {
 }
 
 static double thread_how_many_proc(void*) {
+	hoc_return_type_code = 1; // integer
 	int i = nrn_how_many_processors();
 	return double(i);
 }
 
 static double sec_in_thread(void*) {
+	hoc_return_type_code = 2; // boolean
 	Section* sec = chk_access();
 	return double(sec->pnode[0]->_nt->id);
 }
@@ -928,6 +950,10 @@ static double nrnbbcorewrite(void*) {
 	return double(nrnbbcore_write());
 }
 
+static double nrncorerun(void*) {
+	return double(nrncore_run(gargstr(1)));
+}
+
 static double nrnbbcore_register_mapping(void*) {
 	return double(nrnbbcore_register_mapping());
 }
@@ -1021,6 +1047,7 @@ static Member_func members[] = {
 
     "nrnbbcore_write", nrnbbcorewrite,
     "nrnbbcore_register_mapping", nrnbbcore_register_mapping,
+    "nrncore_run", nrncorerun,
 
 	0,0
 };
@@ -1062,7 +1089,7 @@ void ParallelContext_reg() {
 		retobj_members, retstr_members);
 }
 
-char* BBSImpl::execute_helper(size_t* size, int id) {
+char* BBSImpl::execute_helper(size_t* size, int id, bool exec) {
 	char* s;
 	int subworld = (nrnmpi_numprocs > 1 && nrnmpi_numprocs_bbs < nrnmpi_numprocs_world);
 	int style = upkint();
@@ -1205,11 +1232,16 @@ hoc_execerror("ParallelContext execution error", 0);
 				pickle_ret_ = 0;
 				pickle_ret_size_ = 0;
 			}
-			rs = (*nrnpy_callpicklef)(s, npickle, narg, size);
+			if (exec) {
+			  rs = (*nrnpy_callpicklef)(s, npickle, narg, size);
+			}
 			hoc_ac_ = 0.;
 		}else{
 //printf("%d exec hoc call %s narg=%d\n", nrnmpi_myid_world, fname->name, narg);
-			hoc_ac_ = hoc_call_objfunc(fname, narg, ob);
+			hoc_ac_ = 0.;
+			if (exec) {
+			  hoc_ac_ = hoc_call_objfunc(fname, narg, ob);
+			}
 //printf("%d exec return from hoc call %s narg=%d\n", nrnmpi_myid_world, fname->name, narg);
 		}
 		delete [] s;
